﻿using System.Linq;

namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc.StringVector {
  using System.Collections.Generic;

  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Data;
  using HeuristicLab.Parameters;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
  using HeuristicLab.Random;

  [StorableClass]
  public class StringVectorConstantsErcValue : WeightedErcValueItem<IReadOnlyList<string>> {
    private const string ConstantsParameterName = "Constants";

    public StringVectorConstantsErcValue() : this(false, 1d) { }
    public StringVectorConstantsErcValue(params StringArray[] arrays) : this(true, 1d, arrays) { }

    public StringVectorConstantsErcValue(bool isEnabled, double weight = 1d, params StringArray[] arrays) : base(isEnabled, weight) {
      Name = "String vector constants";
      IsEnabled = isEnabled;
      Parameters.Add(new ValueParameter<ItemCollection<StringArray>>(ConstantsParameterName, new ItemCollection<StringArray>(arrays)));
    }

    [StorableConstructor]
    private StringVectorConstantsErcValue(bool deserializing) : base(deserializing) { }

    public StringVectorConstantsErcValue(StringVectorConstantsErcValue origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<ItemCollection<StringArray>> ConstantsParameter
    {
      get { return (IValueParameter<ItemCollection<StringArray>>)Parameters[ConstantsParameterName]; }
    }

    public IEnumerable<StringArray> Constants
    {
      get { return ConstantsParameter.Value; }
    }

    public override IReadOnlyList<string> GetErcValue(IRandom random) {
      // TODO: remove ToArray when ValueTypeArray implements IReadOnlyList
      return Constants.SampleRandom(random).ToArray();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new StringVectorConstantsErcValue(this, cloner);
    }
  }
}
